% Multidimensional analog extremum seeking control algorithms
% Delft Biorobotics Laboratory, Biomechanical Department,
% Delft University of Technology
% b.calli@tudelft.nl

clear all; clc;  close all;

%% Simulation parameters
stepsize = 0.001;
starttime = 0;
stoptime = 4;
% end of simulation parameters
%% Selections

switch_optimizer = 4;       % 1: None
                            % 2: Neural Network ESC
                            % 3: Approximation based ESC
                            % 4: Perturbation based ESC

switch_noise = 1;           % 1: No noise
                            % 2: With noise in the amount of noise_max
switch_system_model = 1;    % 1: No dynamics
                            % 2: With dynamics using puma560 robot model

time_plot = 0;              % 1: To see the motion of the system on the objective value plot
                            % 2: Plots without motion                            
                            
switch_save = 2;            % 1: Saving the parameters and the plots
                            % 2: No saving


% end of selections
%% output parameters

noise_max = 0.01;

% end of output parameters
%% Initial Values
x0 = [0;0];
% end of initial values
%% robot decleration

puma560;
Kp = 3000;      % control parameters for the robot.
Ki = 0;
Kd = 60;

% end of robot decleration
%% Neural Network Based optimizer parameters

nn.M = 30;            % Gain of the minimum peak detector
nn.A = 0.7;           % Magnitude of the velocity reference output
nn.ddelta1 = 0.02;    % Hysterisis 1 on the velocity reference direction
nn.ddelta2 = 0.025;   % Hysterisis 2 on the velocity reference direction
nn.ddelta3 = 0.03;    % Hysterisis 3 on the velocity reference direction
nn.delta = 0.04;      % Hysterisis of the W switch
nn.B = 600;           % Gain of the W switch
nn.mpd_init = obj_func_multi(x0(1),x0(2));       % initializing the minimum peak detector
% end of Neural Network Based optimizer parameters
%% Interpolation based optimizer
approx.data_size = 10;          % data size to be used for the appoximation
approx.K_grad = 0.6;            % gain of the gradient descent
approx.sampling = 1;            % sampling period of the approximation (unit: cycle, 1: run approximation every cycle, 2: run in every two cycles etc.)
approx.vel_init = 0.1;          % initial velocity reference while collecting the first data set for the approximation
approx.filter_constant = 1;     % lowpass filter constant for the output (LPF: y[k] = lpf_constant * y[k-1] + ( 1.0 - lpf_constant ) * x)

% end of interpolation based optimizer
%% perturbation based optimizer parameters
perturb.sine_amp = 0.07; % amplitudes of the perturbation signals
perturb.sine_freq = 15;    % frequencies of the perturbation signals
perturb.corr_gain = -700;       % adaptation gain
perturb.low_pass_pole = 1;      % pole for the low pass filter
perturb.high_pass_pole = 5;     % pole of the highpass filter
perturb.comp_pole = 500;        % pole of the compensator
perturb.comp_zero = 40;         % zero of the compensator
perturb.sine_phase = pi/2;  % phase shifts of the perturbation signals




% end of perturbation based optimizer parameters
%% Running simulation
if(switch_system_model == 1)        % running simulation without dynamics
    sim('ESC_multi_no_dyn.mdl');
elseif (switch_system_model == 2)   % running simulation with dynamics
    sim('ESC_multi_robot.mdl');
end 
% end of running simulation
%% Plotting
s = -1:0.01:1;

if (time_plot == 1)
    [X,Y] = meshgrid(-1:.5:1);
    Z = obj_func_multi(X,Y);
    figure(1)
    mesh(X,Y,Z),hold on;
    [size_signal, dummy] = size(x.signals.values);
    for i = 1:10:size_signal
        plot3( x.signals.values(i,1), x.signals.values(i,2), y.signals.values(i),'*b')
        fprintf('time: %f secs\n',x.time(i));
        pause(0.01);
        clc;
    end
else
    [X,Y] = meshgrid(-2:.05:2);
    [sx1,sx2] = size(X);
    Z = zeros(sx1,sx2);
    for i = 1:sx1
        for j = 1:sx2
            Z(i,j) = obj_func_multi(X(i,j),Y(i,j));
        end
    end
    figure(1)
    mesh(X,Y,Z),hold on;
    plot3( x.signals.values(:,1), x.signals.values(:,2), y.signals.values,'*b');
    axis([-1.5+0.5 1.5+0.5 -1.5+0.2 1.5+0.2 1 2]);
    figure_handle_obj_u = figure(3);
    maximize(figure_handle_obj_u);
    plot(y.time,y.signals.values, 'linewidth',2), axis([0 stoptime 0.8 1.8]),xlabel('time(s)'),ylabel('Objective value'),grid on;
    figure,plot(u_out.time,u_out.signals.values, 'linewidth',2),xlabel('time(s)'),ylabel('Objective value'),grid on;
end
size_ptd = size(peak_time_detector);
for count_peak = 1:size_ptd(1)
    if(peak_time_detector(count_peak)>0)
        break;
    end
end
fprintf('Total Distance Travelled: %f \n',sqrt(x_total(count_peak,1)^2+x_total(count_peak,2)^2))
% end plotting
%% plot saving
if(switch_save == 1)
    str_plot_dir = 'plots_multi\m_';
    str_wo_dyn = 'wod';
    str_wo_noise = 'won';
    str_w_dyn = 'wd';
    str_w_noise = 'wn';
    str_sliding_mode = 'sm';
    str_neural_networks = 'nn';
    str_approximation = 'approx';
    str_perturbation = 'perturb';
    if (switch_optimizer == 1)
        str_method = str_sliding_mode;
    elseif(switch_optimizer == 2)
        str_method = str_neural_networks;
    elseif(switch_optimizer == 3)
        str_method = str_approximation;
    elseif(switch_optimizer == 4)
        str_method = str_perturbation;
    end

    if(switch_noise == 1)
        str_noise = str_wo_noise;
    elseif(switch_noise == 2)
       noise_max_for_str = noise_max/0.01;
        str_noise = strcat(str_w_noise,num2str(noise_max_for_str));
    end
    
    if(switch_system_model == 1)
        str_dyn = str_wo_dyn;
    elseif(switch_system_model == 2)
        str_dyn = str_w_dyn;
    end
    file_name_to_save = strcat(str_plot_dir,str_dyn,'_',str_noise,'_',str_method);
    saveas(figure_handle_obj_u,file_name_to_save,'fig')
    saveas(figure_handle_obj_u,file_name_to_save,'eps')
    save(file_name_to_save);
end