% Unidimensional analog extremum seeking control algorithms 
% Delft Biorobotics Laboratory, Biomechanical Department,
% Delft University of Technology
% b.calli@tudelft.nl

clear all; clc;  close all;

%% Simulation parameters
stepsize = 0.001;
starttime = 0;
stoptime = 4;
% end of simulation parameters
%% Selections

switch_optimizer = 1;       % 1: Sliding mode ESC
                            % 2: Neural Network ESC
                            % 3: Approximation based ESC
                            % 4: Perturbation based ESC

switch_noise = 1;           % 1: No noise
                            % 2: With noise in the amount of noise_max
                            
switch_system_model = 1;    % 1: No dynamics
                            % 2: With dynamics using puma560 robot model
                            
time_plot = 2;              % 1: To see the motion of the system on the objective value plot
                            % 2: Plots without motion
                            
switch_save = 2;            % 1: Saving the parameters and the plots
                            % 2: No saving
                                                        
% end of selections
%% robot decleration

puma560;
robot = p560;

Kp = 3000;      % control parameters for the robot.
Ki = 0;
Kd = 60;

% end of robot decleration
%% Objective function variables

obj_mean = -0.4;
obj_var = 0.27;

% end of objective function
%% output parameters
noise_max = 0.20;
% end of output parameters
%% Initial Values
x0 = 0;     %system initial position
% end of initial values
%% Sliding mode ESC parameters
% Constraints on parameters:
sm_periodic.rho = 0.0015;   % speed of the driving input
sm_periodic.k = 0.5;        % magnitude of the velocity reference output
sm_periodic.alpha = 0.32;   % parameter for the allowed error bound between driving input and objective value

sm_periodic.g_init = obj_func_uni(x0,obj_mean, obj_var);    % initializing the driving input

% end of Sliding mode optimizer parameters
%% Neural Network ESC parameters
% bi sonrakinde M'yi arttirmayi dene
nn.M = 10;         % Gain of the minimum peak detector
nn.ddelta = 0.03;  % Hysterisis on the velocity reference direction
nn.A= 1;           % Magnitude of the velocity reference output
nn.delta = 0.05;   % Hysterisis of the W switch
nn.B = 30;         % Gain of the W switch
nn.mpd_init = obj_func_uni(x0,obj_mean, obj_var);   % initializing the minimum peak detector
% end of Neural Network Based optimizer parameters
%% Approximation based ESC parameters

approx.data_size = 4;           % data size to be used for the appoximation
approx.poly_degree = 3;         % degree of the polynomial to be approximated
approx.K_grad = 0.29;           % gain of the gradient descent
approx.sampling = 1;            % sampling period of the approximation (unit: cycle, 1: run approximation every cycle, 2: run in every two cycles etc.)
approx.vel_init = 0.1;          % initial velocity reference while collecting the first data set for the approximation
approx.filter_constant = 1;     % lowpass filter constant for the output (LPF: y[k] = lpf_constant * y[k-1] + ( 1.0 - lpf_constant ) * x)

% end of approximation based optimizer parameters
%% perturbation based optimizer parameters

perturb.high_pass_pole = 5;     % pole of the highpass filter
perturb.sine_amp = 0.05;        % amplitude of the perturbation signal
perturb.sine_freq = 15;         % frequency of the perturbation signal
perturb.low_pass_pole = 1;      % pole for the low pass filter
perturb.corr_gain = -500;       % adaptation gain
perturb.comp_zero = 40;         % zero of the compensator
perturb.comp_pole = 700;        % pole of the compensator

% end of perturbation based optimizer parameters
%% Running simulation
if(switch_system_model == 1)        % running simulation without dynamics
    sim('ESC_uni_no_dyn.mdl');
elseif (switch_system_model == 2)   % running simulation with dynamics
    sim('ESC_uni_robot.mdl');
end
% end of running simulation
%% Plotting
s = (-1-0.4):0.01:(1-0.4);

if (time_plot == 1)
    figure(2),plot(s,obj_func_uni(s,obj_mean, obj_var)),hold on
    [size_signal, dummy] = size(x.signals.values);
    for i = 1:10:size_signal
        plot(x.signals.values(i),y.signals.values(i),'*r');
        fprintf('time: %f secs\n',x.time(i))
        pause(0.01)
        clc
        
    end
else
    figure(2),plot(s,obj_func_uni(s,obj_mean, obj_var)),hold on, plot(x.signals.values,y.signals.values,'*r'),grid on;
    figure_handle_obj_u = figure(3);
    maximize(figure_handle_obj_u);
    subplot(2,1,1),plot(y.time,y.signals.values, 'linewidth',2), axis([0 stoptime 0.8 2.2]),xlabel('time(s)'),ylabel('Objective value'),grid on;
    subplot(2,1,2),plot(u_out.time,u_out.signals.values, 'linewidth',2),xlabel('time(s)'), ylabel('Generated velocity reference'),axis([0 stoptime -1.1 1.1]),grid on;
    if (switch_optimizer == 1)
        figure(5),plot(g_sm_out.time,g_sm_out.signals.values),hold on, plot(y.time,y.signals.values,'--r'),legend('g','y'),grid on;
    elseif (switch_optimizer == 2)
        figure(5),plot(yd_out.time,yd_out.signals.values),hold on, plot(y.time,y.signals.values,'--r'),legend('g','y'),grid on;
    elseif (switch_optimizer == 3)
    elseif (switch_optimizer == 4)
    end
    
end
size_ptd = size(peak_time_detector);
for count_peak = 1:size_ptd(1)
    if(peak_time_detector(count_peak)>0)
        break;
    end
end
fprintf('Total Distance Travelled: %f \n',x_total(count_peak))
% END OF plotting
%% saving

str_plot_dir = 'plots_uni\u_';
str_wo_dyn = 'wod';
str_wo_noise = 'won';
str_w_dyn = 'wd';
str_w_noise = 'wn';
str_sliding_mode = 'sm';
str_neural_networks = 'nn';
str_approx = 'approx';
str_perturbation = 'perturb';
str_interpolation = 'inter';

if (switch_save == 1)
    if (switch_optimizer == 1)
        str_method = str_sliding_mode;
    elseif(switch_optimizer == 2)
        str_method = str_neural_networks;
    elseif(switch_optimizer == 3)
        str_method = str_approx;
    elseif(switch_optimizer == 4)
        str_method = str_perturbation;
    elseif(switch_optimizer == 5)
        str_method = str_interpolation;
    end

    if(switch_noise == 1)
        str_noise = str_wo_noise;
    elseif(switch_noise == 2)
        noise_max_for_str = noise_max/0.01;
        str_noise = strcat(str_w_noise,num2str(noise_max_for_str));
    end
    
    if(switch_system_model == 1)
        str_dyn = str_wo_dyn;
    elseif(switch_system_model == 2)
        str_dyn = str_w_dyn;
    end
    file_name_to_save = strcat(str_plot_dir,str_dyn,'_',str_noise,'_',str_method);
    saveas(figure_handle_obj_u,file_name_to_save,'fig')
    saveas(figure_handle_obj_u,file_name_to_save,'eps')
    save(file_name_to_save)    
end
% end of plot saving